/* Copyright (C) 1998-2000  Matthes Bender  RedWolf Design */

/* All kinds of valuable helpers */

#ifndef INC_STANDARD
#define INC_STANDARD

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif //HAVE_CONFIG_H

// debug memory management
#if defined(_DEBUG) && defined(_MSC_VER)
#include <new>
#include <memory>
#include <crtdbg.h>
#define new new(_NORMAL_BLOCK, __FILE__, __LINE__)
#endif

// Integer dataypes
#ifdef HAVE_STDINT_H
#include <stdint.h>
#else
typedef signed __int8 int8_t;
typedef signed __int16 int16_t;
typedef signed __int32 int32_t;
typedef signed __int64 int64_t;
typedef unsigned __int8 uint8_t;
typedef unsigned __int16 uint16_t;
typedef unsigned __int32 uint32_t;
typedef unsigned __int64 uint64_t;
#endif

#include <string.h>

#ifdef _WIN32
#define _WIN32_WINDOWS 0x0500
#define _WIN32_WINNT  0x0501
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#else
// Boolean datatype
typedef int            BOOL;
#define TRUE  ((BOOL)1)
#define FALSE ((BOOL)0)

// Windows integer types
typedef uint32_t       DWORD;
typedef uint8_t        BYTE;
typedef uint16_t       WORD;
typedef uint32_t       UINT;

typedef struct {
    long left; long top; long right; long bottom;
} RECT;

#define INFINITE	0xFFFFFFFF
#define FW_BOLD 700

unsigned long timeGetTime(void);

inline int stricmp(const char *s1, const char *s2) {
	return strcasecmp(s1, s2);
}

#define GetRValue(rgb) ((unsigned char)(rgb))
#define GetGValue(rgb) ((unsigned char)(((unsigned short)(rgb)) >> 8))
#define GetBValue(rgb) ((unsigned char)((rgb)>>16))
#define RGB(r,g,b) ((DWORD)((BYTE)(r)|((BYTE)(g) << 8)|((BYTE)(b) << 16)))
#define ZeroMemory(d,l) memset((d), 0, (l))
#endif

#include <memory.h>
#include <math.h>

// Color triplets
#define BGR(b,g,r) RGB(r,g,b)

// Small helpers
template <class T> inline T Max(T val1, T val2) { return val1 > val2 ? val1 : val2; }
template <class T> inline T Min(T val1, T val2) { return val1 < val2 ? val1 : val2; }
template <class T> inline T Abs(T val) { return val > 0 ? val : -val; }
template <class T> inline bool Inside(T ival, T lbound, T rbound) { return ival >= lbound && ival <= rbound; }
template <class T> inline T BoundBy(T bval, T lbound, T rbound) { return bval < lbound ? lbound : bval > rbound ? rbound : bval; }
template <class T> inline int Sign(T val) { return val < 0 ? -1 : val > 0 ? 1 : 0; }
template <class T> inline void Swap(T &v1, T &v2) { T t = v1; v1 = v2; v2 = t; }
template <class T> inline void Toggle(T &v) { v = !v; }

inline int sqrt(int num) { return (int)sqrt(double(num)); }
const double pi = 3.14159265358979323846;

inline void DWordAlign(int &val)
  {
  if (val%4) { val>>=2; val<<=2; val+=4; }
  }

inline int DWordAligned(int val)
  {
  if (val%4) { val>>=2; val<<=2; val+=4; }
	return val;
  }

inline int QWordAligned(int val)
  {
  if (val%8) { val>>=3; val<<=3; val+=8; }
	return val;
  }


inline int32_t ForceLimits(int32_t &rVal, int32_t iLow, int32_t iHi)
	{
	if (rVal<iLow) { rVal=iLow; return -1; }
	if (rVal>iHi)  { rVal=iHi;  return +1; }
	return 0;
	}

int Distance(int iX1, int iY1, int iX2, int iY2);
int Angle(int iX1, int iY1, int iX2, int iY2);

inline void FillMem(void *lpMem, size_t dwSize, char bValue)
  {
	memset(lpMem,bValue,dwSize);
  }

inline void ZeroMem(void *lpMem, size_t dwSize)
  {
	FillMem(lpMem,dwSize,0);
  }

inline bool MemEqual(const void *lpMem1, const void *lpMem2, size_t dwSize)
  {
	return !memcmp(lpMem1,lpMem2,dwSize);
  }

inline void MemCopy(const void *lpMem1, void *lpMem2, size_t dwSize)
  {
	memmove(lpMem2,lpMem1,dwSize);
	}

bool ForLine(int32_t x1, int32_t y1, int32_t x2, int32_t y2, 
             bool (*fnCallback)(int32_t, int32_t, int32_t), int32_t iPar=0,
						 int32_t *lastx=NULL, int32_t *lasty=NULL);

bool PathMove(int &rX, int &rY, int iTargetX, int iTargetY, int iSteps, int &rDelta);

char CharCapital(char cChar);
bool IsIdentifier(char cChar);
bool IsUpperCase(char cChar);
bool IsLowerCase(char cChar);
bool IsWhiteSpace(char cChar);

int  SLen(const char *sptr);
int  SLenUntil(const char *szStr, char cUntil);

bool SEqual(const char *szStr1, const char *szStr2);
bool SEqual2(const char *szStr1, const char *szStr2);
bool SEqualUntil(const char *szStr1, const char *szStr2, char cWild);
bool SEqualNoCase(const char *szStr1, const char *szStr2, int iLen=-1);
bool SEqual2NoCase(const char *szStr1, const char *szStr2, int iLen = -1);

int  SCompare(const char *szStr1, const char *szStr2);

void SCopy(const char *szSource, char *sTarget, int iMaxL=-1);
void SCopyUntil(const char *szSource, char *sTarget, char cUntil, int iMaxL=-1);
void SCopyUntil(const char *szSource, char *sTarget, const char * sUntil, size_t iMaxL);
void SCopyIdentifier(const char *szSource, char *sTarget, int iMaxL=0);
bool SCopyPrecedingIdentifier(const char *pBegin, const char *pIdentifier, char *sTarget, int iSize);
bool SCopySegment(const char *fstr, int segn, char *tstr, char sepa=';', int iMaxL=-1, bool fSkipWhitespace=false);
bool SCopyNamedSegment(const char *szString, const char *szName, char *sTarget, char cSeparator=';', char cNameSeparator='=', int iMaxL=-1);
bool SCopyEnclosed(const char *szSource, char cOpen, char cClose, char *sTarget, int iSize);

void SAppend(const char *szSource, char *szTarget, int iMaxL=-1);
void SAppendChar(char cChar, char *szStr);

void SInsert(char *szString, const char *szInsert, int iPosition=0, int iMaxLen=-1);
void SDelete(char *szString, int iLen, int iPosition=0);

int  SCharPos(char cTarget, const char *szInStr, int iIndex=0);
int  SCharLastPos(char cTarget, const char *szInStr);
int  SCharCount(char cTarget, const char *szInStr, const char *cpUntil=NULL);
int  SCharCountEx(const char *szString, const char *szCharList);

void SReplaceChar(char *str, char fc, char tc);

const char *SSearch(const char *szString, const char *szIndex);
const char *SSearchNoCase(const char *szString, const char *szIndex);
const char *SSearchIdentifier(const char *szString, const char *szIndex);
const char *SSearchFunction(const char *szString, const char *szIndex);

const char *SAdvanceSpace(const char *szSPos);
const char *SAdvancePast(const char *szSPos, char cPast);

bool SGetModule(const char *szList, int iIndex, char *sTarget, int iSize=-1);
bool SIsModule(const char *szList, const char *szString, int *ipIndex=NULL, bool fCase=false);
bool SAddModule(char *szList, const char *szModule);
bool SAddModules(char *szList, const char *szModules);
bool SRemoveModule(char *szList, const char *szModule, bool fCase=false);
int SModuleCount(const char *szList);

void SRemoveComments(char *szScript);
void SNewSegment(char *szStr, const char *szSepa=";");
void SCapitalize(char *szString);
void SWordWrap(char *szText, char cSpace, char cSepa, int iMaxLine);
int  SClearFrontBack(char *szString, char cClear=' ');

char *SGetFilename(const char *szFilepath);

// case sensitive wildcard match with some extra functionality
// can match strings like  "*Cl?nk*vour" to "Clonk Endeavour"
BOOL SWildcardMatchEx(const char *szString, const char *szWildcard);

#define LineFeed "\x00D\x00A"
#define EndOfFile "\x020"

void BufferBlit(void *bypSource, int iSourcePitch, 
                int iSrcBufHgt, // Positive: Bottom up
                int iSrcX, int iSrcY, int iSrcWdt, int iSrcHgt,
                void *bypTarget, int iTargetPitch, 
                int iTrgBufHgt, // Positive: Bottom up
                int iTrgX, int iTrgY, int iTrgWdt, int iTrgHgt);

void BufferBlitDw(uint32_t *bypSource, int iSourcePitch, 
                int iSrcBufHgt, // Positive: Bottom up
                int iSrcX, int iSrcY, int iSrcWdt, int iSrcHgt,
                uint32_t *bypTarget, int iTargetPitch, 
                int iTrgBufHgt, // Positive: Bottom up
                int iTrgX, int iTrgY, int iTrgWdt, int iTrgHgt);

void BufferBlitAspect(void *bypSource, int iSourcePitch, 
                int iSrcBufHgt, // Positive: Bottom up
                int iSrcX, int iSrcY, int iSrcWdt, int iSrcHgt,
                void *bypTarget, int iTargetPitch, 
                int iTrgBufHgt, // Positive: Bottom up
                int iTrgX, int iTrgY, int iTrgWdt, int iTrgHgt);

void BufferBlitAspectDw(uint32_t *bypSource, int iSourcePitch, 
                int iSrcBufHgt, // Positive: Bottom up
                int iSrcX, int iSrcY, int iSrcWdt, int iSrcHgt,
                uint32_t *bypTarget, int iTargetPitch, 
                int iTrgBufHgt, // Positive: Bottom up
                int iTrgX, int iTrgY, int iTrgWdt, int iTrgHgt);

void StdBlit(uint8_t *bypSource, int iSourcePitch, int iSrcBufHgt,
							int iSrcX, int iSrcY, int iSrcWdt, int iSrcHgt,
							uint8_t *bypTarget, int iTargetPitch, int iTrgBufHgt,
							int iTrgX, int iTrgY, int iTrgWdt, int iTrgHgt,
							int iBytesPerPixel=1, bool fFlip=false);


#endif // INC_STANDARD

