/* Copyright (C) 1998-2000  Matthes Bender  RedWolf Design */

/* Scan ini-type text component files to run-time data structures */

#include <C4Include.h>
#include <C4Compiler.h>
#ifdef C4ENGINE
#include <C4Object.h>
#endif

const int32_t MaxRefBuf	 = 100,
					MaxLineBuf = 4096,
					MaxCompileBuf = 20000;

BOOL C4Compiler::CompileLine(C4CompilerValue *pRefs, 
														 const char *szSection, 
														 const char *szLine, 
														 BYTE *pTarget, 
														 const char *szSecExt,
														 const char *szLanguage,
														 int32_t iSectionMask)
	{
	C4CompilerValue *cRef;
	char szRefName[MaxRefBuf+1];
  char szSegment[MaxRefBuf+1];
	int32_t cnt;

	if (!pRefs || !szSection || !szLine || !pTarget) return FALSE;

	// No good value line
	if (!SCharCount('=',szLine)) return FALSE;
	// Get ref name
	SCopyUntil(szLine,szRefName,'=',MaxRefBuf);
	szLine = SAdvancePast(szLine,'=');
	// Find reference in correct section
	char csec[MaxRefBuf+1]; csec[0]=0; int32_t iSecMask;
	for (cRef=pRefs; cRef->Type!=C4CV_End; cRef++)
		{
		// New section
		if (cRef->Type==C4CV_Section)
			{
			sprintf(csec,cRef->Name,szSecExt);
			iSecMask = cRef->Offset;
			}
		// Value
		else
			{
			// Match section, match value, match mask
			if (SEqual(szSection,csec) && (!iSectionMask || (iSectionMask & iSecMask)))
				{

//          -- disabled, has problems with language priorities (and seems to be unused anyway)
//				// make language dependant
//				char szRefListName[MaxRefBuf+1];
//				SCopy(cRef->Name, szRefListName);
//				int32_t iLen=SLen(szRefName);
//				if (iLen>=2) if (szRefListName[iLen-1]=='*' && szRefListName[iLen-2]=='*') if (szLanguage && SLen(szLanguage)>=2)
//					SCopy(szLanguage, szRefListName+iLen-2, 2);
//				if (SEqual(szRefName,szRefListName))	break;

					if(SEqual(szRefName, cRef->Name)) break;
				}
			}
		}
	if (!cRef || cRef->Type==C4CV_End)
		{
		#ifdef C4ENGINE
		// Warn if in Debugmode and no Game.txt because Game.txt is loaded very strangely
		if (Config.General.DebugMode && !Game.GameText.GetData())
			{
			//sprintf(OSTR,"  Warning: Unknown entry %s in section %s",szRefName,szSection);
			//Log(OSTR);
			}
		#endif
		return FALSE; // Reference not found
		}

	// Convert and set referenced value
	pTarget += cRef->Offset;
	switch (cRef->Type)
		{

		case C4CV_Ignore:
			// Ignore me...
			break;

		case C4CV_String:
			SCopy( szLine, (char*)pTarget, cRef->Size );
			break;

		case C4CV_StringPtr:
			(*(StdStrBuf*) pTarget).Copy(szLine);
			break;
#if 0
		case C4CV_Id:
      for (cnt=0; (cnt<cRef->Size) && SCopySegment(szLine,cnt,szSegment,',',C4MaxName); cnt++)
        if (SEqual(szSegment,"NONE") || (SLen(szSegment)!=4))
          ((C4ID*)pTarget) [cnt] = C4ID_None;
        else
          ((C4ID*)pTarget) [cnt] = C4Id(szSegment); 
			break;
#endif
    case C4CV_Integer:
		case C4CV_OldColor:
		case C4CV_Fixed:
      for (cnt=0; (cnt<cRef->Size) && SCopySegment(szLine,cnt,szSegment,',',C4MaxName); cnt++)
				{
				const char *pPos = szSegment; char cNumForm;
				// Fixed -> get num format prefix
				if(cRef->Type == C4CV_Fixed)
				{
					pPos = SAdvanceSpace(pPos);
					cNumForm = *pPos;
					if(cNumForm == 'f' || cNumForm == 'F') pPos++;
				}
				// read the number
				int32_t *pIntTarget = ((int32_t*)pTarget) + cnt;
        sscanf( pPos, "%i",  pIntTarget);
				// conversion
#ifdef C4ENGINE
				if (cRef->Type==C4CV_OldColor) if (Inside<int32_t>(*pIntTarget, 0, C4MaxColor-1))
					*pIntTarget = Application.DDraw->Pal.Colors[FColors[FPlayer+*pIntTarget]];
#endif
				if(cRef->Type == C4CV_Fixed)
#ifdef USE_FIXED
					if(cNumForm	 == 'f') FLOAT_TO_FIXED((FIXED *)pIntTarget); // float
#else
					if(cNumForm != 'f') FIXED_TO_FLOAT((FIXED *)pIntTarget); // C4Fixed
#endif				
				}
      break;
#if 0
    case C4CV_ValueList:

			// clear old
			((C4ValueList*)pTarget)->Reset();
			// set new
			if(SCharPos(';',szLine,0)==-1)
			{
				// old format ( [Value],[Value], etc.)
				for (cnt=0; (cnt<C4MaxVariable) && SCopySegment(szLine,cnt,szSegment,',',C4MaxName); cnt++)
				{
					int32_t nItem=0;
					sscanf( szSegment, "%i", &nItem); 
					((C4ValueList*)pTarget)->SetItem(cnt,C4VInt(nItem));
				}
				break;
			}
			else
			{
				// new format ( [Count];[TypeID][Value],[TypeID][Value], etc.)
				if(!SCopySegment(szLine,0,szSegment,';',C4MaxName)) break;
				// load item count
				int32_t nItemCnt=0;
				sscanf( szSegment , "%i" , &nItemCnt);
				((C4ValueList*)pTarget)->SetSize(nItemCnt);
				// advance to values
				szLine = SAdvancePast( szLine, ';' );
				// load values ( + type info)
				for (cnt=0; (cnt<nItemCnt) && SCopySegment(szLine,cnt,szSegment,',',C4MaxName); cnt++)
					if(!(*(C4ValueList *)pTarget)[cnt].SetValSaveStr(szSegment))
						break;
			}
      break;

		case C4CV_ValueMap:
			{
			// format : [Count];[Name]=[TypeID][Value],[Name]=[TypeID][Value], etc.
			C4ValueMapData* pVMap = (C4ValueMapData*) pTarget;
			pVMap->Reset();
			// read item count
			int32_t nItemCnt=0;
			if(!SCopySegment(szLine,0,szSegment,';',C4MaxName)) break;
			sscanf( szSegment , "%i" , &nItemCnt);
			// advance to values
			szLine = SAdvancePast( szLine, ';' );
			// create arrays to receive names & values
			char** ppNames = new char* [nItemCnt];
			C4Value* pValues = new C4Value [nItemCnt] ();
			for (cnt=0; (cnt<nItemCnt) && SCopySegment(szLine,cnt,szSegment,',',MaxRefBuf); cnt++)
			{
				const char* AktPos = szSegment;

				// alloc buffer for variable name
				ppNames[cnt] = new char [MaxRefBuf + 1];
				// read variable name
				SCopyUntil(AktPos,ppNames[cnt],'=',MaxRefBuf);
				AktPos = SAdvancePast(AktPos,'=');
				// read value
				AktPos = SAdvanceSpace(AktPos);
				pValues[cnt].SetValSaveStr(AktPos); 
			}
			// if the list is incomplete, reset item count to what could be read
			nItemCnt = cnt;
			// *** copy into valuemap
			// clear
			pVMap->ResetContent();
			// no name list?
			if(!pVMap->pNames)
			{
				// create temporary name list
				// (must be later replaced by official name list!)
				pVMap->CreateTempNameList();
				// set name list
				pVMap->pNames->SetNameArray((const char**) ppNames, nItemCnt);
			}
			int32_t i;
			// set data...
			for(i=0;i<nItemCnt;i++)
			{
				C4Value* pZValue = pVMap->GetItem(ppNames[i]);
				if(!pZValue)
				{
#ifdef C4ENGINE
					char strMsg[100+1];
					sprintf(strMsg, "could not find variable \"%s\"!", ppNames[i]);
					Log(strMsg);
#endif
				}
				else
					pZValue->Set(pValues[i]);
			}
			// delete our name & data list
			for(i=0;i<nItemCnt;i++)
				delete[] ppNames[i];
			delete[] ppNames;
			delete[] pValues;
      }
			break;

		case C4CV_NameList:
			((C4NameList*)pTarget)->Read(szLine,0);
			break;

		case C4CV_IdList:
			((C4IDList*)pTarget)->Read(szLine,0);
			break;
#endif
#ifdef C4ENGINE // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

		case C4CV_ObjectList:
			// empty current list
			((C4ObjectList*)pTarget)->Clear();
			// Read string of enumerated pointers for post-compile
			((C4ObjectList*)pTarget)->ReadEnumerated(szLine);
			break;

		case C4CV_DefGraphics:
			{
			// get line length
			int32_t iLineLength = SLen(szLine);
			// not C4ID::Name?
			if (iLineLength < 6 || szLine[4]!=':' || szLine[5]!=':') break;
			// get ID
			char id[5]; SCopy(szLine, id, 4);
			C4Def *pSrcDef = Game.Defs.ID2Def(C4Id(id)); // defaults to NULL for unloaded def
			if (!pSrcDef) break;
			// get graphics - "C4ID::" leads to *szLine == NULL, and thus the default graphic of pSrcDef!
			C4DefGraphics *pGrp = pSrcDef->Graphics.Get(szLine+6);
			// set graphics - doesn't update, and zeroes for unloaded graphics
			*(C4DefGraphics **) pTarget = pGrp;
			}
			break;

		case C4CV_Transform:
			{
			// create new transform
			C4DrawTransform *pNewTransform = new C4DrawTransform(), **ppTrfTarget = reinterpret_cast<C4DrawTransform **>(pTarget);
			// assign values
			int32_t iScanCnt = sscanf(szLine, "%f,%f,%f,%f,%f,%f,%d",
				&pNewTransform->mat[0], &pNewTransform->mat[1], &pNewTransform->mat[2],
				&pNewTransform->mat[3], &pNewTransform->mat[4], &pNewTransform->mat[5],
				&pNewTransform->FlipDir
				);
			// validate
			if (iScanCnt != 7)
				{
				sprintf(OSTR, "malformed C4CV_Transform: %s", szLine); Log(OSTR);
				delete pNewTransform;
				break;
				}
			// delete any previous transform
			if (*ppTrfTarget) delete *ppTrfTarget;
			// assign new
			*ppTrfTarget = pNewTransform;
			}
			break;

		case C4CV_EffectList:
			{
			C4Effect **ppEffTarget = reinterpret_cast<C4Effect **>(pTarget);
			// delete any previous effects
			if (*ppEffTarget) { delete *ppEffTarget; *ppEffTarget=NULL; }
			while (*szLine)
				{
				// read effect (this advances the szLine-ptr)
				*ppEffTarget = new C4Effect(&szLine);
				ppEffTarget = &(*ppEffTarget)->pNext;
				// skip seperator (whatever it is)
				if (*szLine) ++szLine;
				}
			}
			break;

		case C4CV_GfxOverlayList:
			{
			C4GraphicsOverlay **ppOvrlTarget = reinterpret_cast<C4GraphicsOverlay **>(pTarget);
			// delete any previous overlays
			if (*ppOvrlTarget) { delete *ppOvrlTarget; *ppOvrlTarget=NULL; }
			while (*szLine)
				{
				// read overlay (this advances the szLine-ptr)
				*ppOvrlTarget = new C4GraphicsOverlay();
				(*ppOvrlTarget)->Read(&szLine);
				ppOvrlTarget = &(*ppOvrlTarget)->pNext;
				// skip seperator (whatever it is)
				if (*szLine) ++szLine;
				}
			}
			break;

#endif // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

		}

	return TRUE;
	}

BOOL C4Compiler::CompileStructure(C4CompilerValue *pRefs, 
																	const char *szSource, 
																	void *vpTarget,
																	const char *szSecExt,
																	const char *szLanguage,
																	C4CompileStructureCallback pfnCallback,
																	int32_t iSectionMask)
	{
	char section[MaxRefBuf+1];
	char valueline[MaxLineBuf+1];
	const char *cpos;

	if (!szSource) return FALSE;
	if ((!pRefs || !vpTarget) && !pfnCallback) return FALSE;

	BYTE *pTarget = (BYTE*) vpTarget;

	section[0]=0;
	cpos=szSource;

	// Scan source text
	while (*cpos)
		{

		// Section
		if (*cpos=='[')
			{
			// Set current section name
			SCopyUntil(cpos+1,section,']',MaxRefBuf);
			// callback to set target
			if (pfnCallback) pfnCallback(section, (void *&)pTarget, pRefs);
			// Advance to next line
			cpos=SAdvancePast(cpos,0x0A);
			}

		// Value
		else if (IsIdentifier(*cpos))
			{
			// Get value line
			//SCopyUntil(cpos,valueline,0x0D,MaxLineBuf);
			SCopyUntil(cpos,valueline,"\r\n",MaxLineBuf);
			// Compile line (if inside known section)
			if (pRefs) CompileLine(pRefs,section,valueline,pTarget,szSecExt,szLanguage);
			// Advance to next line
			cpos=SAdvancePast(cpos,0x0A);
			}

		// No good line
		else
			{
			// Advance to next line
			cpos=SAdvancePast(cpos,0x0A);
			}

		}

	return TRUE;
	}


BOOL C4Compiler::DecompileStructure(C4CompilerValue *pRefs, 
																		void *vpData, 
																		void *vpDefault, 
																		char **ppOutput, 
																		size_t *ipSize,
																		const char *szSecExt,
																		int32_t iSectionMask)
	{
	char ostr[MaxLineBuf+1],ostr2[MaxLineBuf+1];
	char secname[MaxRefBuf+1]="[Unknown]"; BOOL secput=FALSE;
	int32_t cnt;
	BOOL fDiff; int32_t iDiff;
	BOOL fOwnBuffer=FALSE;
	
	if (!pRefs || !vpData || !vpDefault) return FALSE;
	
	BYTE *pData = (BYTE*) vpData;
	BYTE *pDefault = (BYTE*) vpDefault;

	// No buffer provided: create output buffer (static max size)
	if (!(*ppOutput)) 
		{
		*ppOutput = new char [MaxCompileBuf];
		*ppOutput[0]=0;
		fOwnBuffer=TRUE;
		}

	// Create output text by processing all variable refs
	for (C4CompilerValue* cRef=pRefs; cRef->Type!=C4CV_End; cRef++)
		switch (cRef->Type)
			{

			case C4CV_Section:
				// check mask
				if (iSectionMask && !(iSectionMask & cRef->Offset))
					{
					// mask not matching: search for next section or end
					while (cRef->Type != C4CV_End && (cRef->Type != C4CV_Section || !(cRef->Offset & iSectionMask)))
						++cRef;
					if (cRef->Type == C4CV_End) break;
					}
				// Prepare section name
				secname[0]=0;
				if (*ppOutput[0]) SAppend(LineFeed,secname);
				sprintf(ostr,"[%s]",cRef->Name);
				sprintf(ostr2,ostr,szSecExt);
				SAppend(ostr2,secname);
				SAppend(LineFeed,secname);
				secput=FALSE;
				break;

			case C4CV_String:
				// Compare to default
				if (SEqual( (char*)(pData+cRef->Offset), (char*)(pDefault+cRef->Offset) )) break;
				// Section name check
				if (!secput) { SAppend(secname,*ppOutput); secput=TRUE; }
				// Append to output
				SAppend(cRef->Name,*ppOutput); SAppend("=",*ppOutput);
				SAppend((char*)(pData+cRef->Offset),*ppOutput);
				SAppend(LineFeed,*ppOutput);
				break;

			case C4CV_StringPtr:
				// Not NULL
				if (!*(StdStrBuf*)(pData+cRef->Offset)) break;
				// Compare to default (later, when there is something with default != NULL)
				//if (SEqual (((C4SimpleString*)(pData+cRef->Offset))->s, ((C4SimpleString*)(pDefault+cRef->Offset))->s)) break;
				// Section name check
				if (!secput) { SAppend(secname,*ppOutput); secput=TRUE; }
				// Append to output
				SAppend(cRef->Name,*ppOutput); SAppend("=",*ppOutput);
				SAppend(*((StdStrBuf*)(pData+cRef->Offset)),*ppOutput);
				SAppend(LineFeed,*ppOutput);
				break;
#if 0
			case C4CV_Id:
				// Access data
				C4ID *idpData,*idpDefault; 
				idpData = (C4ID*) (pData+cRef->Offset);
				idpDefault = (C4ID*) (pDefault+cRef->Offset);
				// Compare to default
				fDiff = FALSE;
				for (cnt=0; cnt<cRef->Size; cnt++)
					if ( idpData[cnt] != idpDefault[cnt] ) 
						fDiff = TRUE;
				if (!fDiff) break;
				// Section name check
				if (!secput) { SAppend(secname,*ppOutput); secput=TRUE; }
				// Append to output
				SAppend(cRef->Name,*ppOutput); SAppend("=",*ppOutput);
				for (cnt=0; cnt<cRef->Size; cnt++)
					{
					if (cnt) SAppend(",",*ppOutput);
					GetC4IdText(idpData[cnt],ostr);
					SAppend(ostr,*ppOutput);
					}
				SAppend(LineFeed,*ppOutput);
				break;
#endif
			case C4CV_Integer:
			case C4CV_Fixed:
				// Access data
				int32_t *ipData,*ipDefault; 
				ipData = (int32_t*) (pData+cRef->Offset);
				ipDefault = (int32_t*) (pDefault+cRef->Offset);
				// Compare to default
				iDiff = 0;
				for (cnt=0; cnt<cRef->Size; cnt++)
					if ( ipData[cnt] != ipDefault[cnt] ) 
						iDiff = cnt+1;
				if (!iDiff) break;
				// Section name check
				if (!secput) { SAppend(secname,*ppOutput); secput=TRUE; }
				// Append to output
				SAppend(cRef->Name,*ppOutput); SAppend("=",*ppOutput);
				for (cnt=0; cnt<iDiff; cnt++)
					{
					if (cnt) SAppend(",",*ppOutput);
					if (cRef->Type != C4CV_Fixed)
						sprintf(ostr, "%i", ipData[cnt]);
					else
#ifdef USE_FIXED
						sprintf(ostr, "F%i", ipData[cnt]);
#else
						sprintf(ostr, "f%i", ipData[cnt]);
#endif
					SAppend(ostr,*ppOutput);
					}
				SAppend(LineFeed,*ppOutput);
				break;
#if 0
			case C4CV_ValueList:
			{
				// Access data
				C4ValueList *iLpData,*iLpDefault; 
				iLpData = (C4ValueList*) (pData+cRef->Offset);
				iLpDefault = (C4ValueList*) (pDefault+cRef->Offset);
				// Compare to default
				if ( *iLpData == *iLpDefault ) break;
				// Section name check
				if (!secput) { SAppend(secname,*ppOutput); secput=TRUE; }
				// Append to output
				SAppend(cRef->Name,*ppOutput); SAppend("=",*ppOutput);
				sprintf(ostr,"%i",iLpData->getSize());
				SAppend(ostr,*ppOutput); SAppend(";",*ppOutput);
				for (cnt=0; cnt<iLpData->getSize(); cnt++)
					{
					if (cnt) SAppend(",",*ppOutput);
					SAppend(iLpData->getItem(cnt).GetValSaveStr(),*ppOutput);
					}
				SAppend(LineFeed,*ppOutput);
				break;
			}

			case C4CV_ValueMap:
			{
				// Access data
				C4ValueMapData *iMpData; 
				iMpData = (C4ValueMapData*) (pData+cRef->Offset);
				// empty?
				if ( iMpData->GetAnzItems() == 0 ) break;
				// Section name check
				if (!secput) { SAppend(secname,*ppOutput); secput=TRUE; }
				// Append to output
				SAppend(cRef->Name,*ppOutput); SAppend("=",*ppOutput);
				// append item count
				sprintf(ostr,"%i",iMpData->GetAnzItems());
				SAppend(ostr,*ppOutput); SAppend(";",*ppOutput);
				// and the items themselves
				for (cnt=0; cnt<iMpData->GetAnzItems(); cnt++)
					{
					if (cnt) SAppend(",",*ppOutput);				
					sprintf(ostr,"%s=%s",iMpData->pNames->pNames[cnt],iMpData->GetItem(cnt)->GetValSaveStr());
					SAppend(ostr,*ppOutput);
					}
				SAppend(LineFeed,*ppOutput);
				break;
			}

			case C4CV_NameList:
				// Access data
				C4NameList *npData,*npDefault; 
				npData = (C4NameList*) (pData+cRef->Offset);
				npDefault = (C4NameList*) (pDefault+cRef->Offset);
				// Compare to default
				if ( *npData == *npDefault ) break;
				// Section name check
				if (!secput) { SAppend(secname,*ppOutput); secput=TRUE; }
				// Append to output
				SAppend(cRef->Name,*ppOutput); SAppend("=",*ppOutput);
				npData->Write(ostr);
				SAppend(ostr,*ppOutput);
				SAppend(LineFeed,*ppOutput);
				break;

			case C4CV_IdList:
				// Access data
				C4IDList *ilpData,*ilpDefault; 
				ilpData = (C4IDList*) (pData+cRef->Offset);
				ilpDefault = (C4IDList*) (pDefault+cRef->Offset);
				// Compare to default
				if ( *ilpData == *ilpDefault ) break;
				// Section name check
				if (!secput) { SAppend(secname,*ppOutput); secput=TRUE; }
				// Append to output
				SAppend(cRef->Name,*ppOutput); SAppend("=",*ppOutput);
				ilpData->Write(ostr);
				SAppend(ostr,*ppOutput);
				SAppend(LineFeed,*ppOutput);
				break;
#endif
#ifdef C4ENGINE

			case C4CV_ObjectList:
				// Access data
				C4ObjectList *olpData; 
				olpData = (C4ObjectList*) (pData+cRef->Offset);
				// Check empty
				if (olpData->IsClear()) break;
				// Section name check
				if (!secput) { SAppend(secname,*ppOutput); secput=TRUE; }
				// Append to output
				SAppend(cRef->Name,*ppOutput); SAppend("=",*ppOutput);
				olpData->Write(ostr);
				SAppend(ostr,*ppOutput);
				SAppend(LineFeed,*ppOutput);
				break;

			case C4CV_DefGraphics:
				{
				// only valid in C4Object - get object
				C4Object *pObj = (C4Object *) pData;
				// get graphics - same as C4Object->pGraphics
				C4DefGraphics *pGrp = *(C4DefGraphics **) (pData+cRef->Offset);
				if (!pGrp) break;
				// check if it's the default
				if (pGrp == &pObj->Def->Graphics) break;
				// Section name check
				if (!secput) { SAppend(secname,*ppOutput); secput=TRUE; }
				// append C4ID::Graphicsname (or C4ID:: for def graphics)
				SAppend(cRef->Name,*ppOutput); SAppend("=",*ppOutput);
				SAppend(C4IdText(pGrp->pDef->id), *ppOutput);
				SAppend("::", *ppOutput);
				const char *szGrpName = pGrp->GetName();
				if (szGrpName) SAppend(szGrpName, *ppOutput);
				SAppend(LineFeed,*ppOutput);
				}
				break;

			case C4CV_Transform:
				{
				// get transform field
				C4DrawTransform *pTarget = *reinterpret_cast<C4DrawTransform **>(pData+cRef->Offset);
				// assigned?
				if (!pTarget) break;
				// Section name check
				if (!secput) { SAppend(secname,*ppOutput); secput=TRUE; }
				// write data
				// CAUTION: If this is changed to StdCompiler, remember to retain the floating point format
				//  or provide a compatibility interface for old Objects.txt using these transforms!
				sprintf(ostr, "%s=%f,%f,%f,%f,%f,%f,%d%s", cRef->Name,
					pTarget->mat[0], pTarget->mat[1], pTarget->mat[2],
					pTarget->mat[3], pTarget->mat[4], pTarget->mat[5], pTarget->FlipDir, LineFeed);
				SAppend(ostr,*ppOutput);
				}
				break;

			case C4CV_EffectList:
				{
				// get first effect
				C4Effect *pEffect = *reinterpret_cast<C4Effect **>(pData+cRef->Offset);
				if (!pEffect) break;
				// Section name check
				if (!secput) { SAppend(secname,*ppOutput); secput=TRUE; }
				SAppend(cRef->Name, *ppOutput);
				SAppend("=", *ppOutput);
				// write data for all effects, seperated by ','
				for (int32_t i=0; pEffect; pEffect = pEffect->pNext)
					{
					if (pEffect->IsDead()) continue;
					if (i++) SAppend(",", *ppOutput);
					pEffect->Write(*ppOutput);
					}
				// terminate line
				SAppend(LineFeed,*ppOutput);
				}
				break;

			case C4CV_GfxOverlayList:
				{
				// get first overlay
				C4GraphicsOverlay *pOverlay = *reinterpret_cast<C4GraphicsOverlay **>(pData+cRef->Offset);
				if (!pOverlay) break;
				// Section name check
				if (!secput) { SAppend(secname,*ppOutput); secput=TRUE; }
				SAppend(cRef->Name, *ppOutput);
				SAppend("=", *ppOutput);
				// write data for all overlays, seperated by ','
				for (int32_t i=0; pOverlay; pOverlay = pOverlay->GetNext())
					{
					if (i++) SAppend(",", *ppOutput);
					pOverlay->Write(*ppOutput);
					}
				// terminate line
				SAppend(LineFeed,*ppOutput);
				}
				break;

#endif // C4ENGINE

			}

	// Self-allocated buffer: finish with eof
	if (fOwnBuffer)
		{
		SAppend(EndOfFile,*ppOutput);
		*ipSize = SLen(*ppOutput);
		}

	return TRUE;
	}
	
