// Simple C4MaterialMap for use with standalone C4MapCreatorS2

#include <stdio.h>
#include <stdexcept>
#include <Standard.h>
#include <C4Constants.h>
#include <C4Material.h>
#include <C4Components.h>
#include <C4Compiler.h>

#define offsC4M(x) offsetof(C4Material, x)

C4CompilerValue C4CR_Material[] = {
	{ "Material", C4CV_Section, 0, 0 },
	{ "Name", C4CV_String, offsC4M(Name), C4M_MaxName },
	{ "Color", C4CV_Integer, offsC4M(Color), C4M_ColsPerMat*3 }
};

void C4Material::Compile(const char* file, const char* source)
{
	memset(this, 0, sizeof(C4Material) );
	C4Compiler Compiler;
	if(Compiler.CompileStructure(C4CR_Material, source, this) == FALSE)
		throw std::runtime_error(std::string("Failed to compile material definition file ") + file);

	if(Name[0] == '\0')
		throw std::runtime_error(std::string("Material definition file ") + file + " has no name");
}

void C4Material::Load(C4Group& hGroup, const char* entryname)
{
	uint8_t* source;
	if(!hGroup.LoadEntry(entryname, &source, NULL, 1) )
		throw std::runtime_error(std::string("Failed to load ") + entryname + " from " + hGroup.GetName() + ": " + hGroup.GetError() );
	
	try
	{
		Compile(entryname, (const char*)source);
	}
	catch(...)
	{
		delete[] source;
		throw;
	}

	delete[] source;
}

C4MaterialMap::C4MaterialMap(C4Group& hGroup, C4Group* OverloadFile)
 : Num(0), Map(NULL)
{
	load_group(hGroup, OverloadFile);
}

C4MaterialMap::~C4MaterialMap()
{
	delete[] Map;
}

void C4MaterialMap::load_group(C4Group& hGroup, C4Group* OverloadFile)
{
	uint8_t* filebuf;
	char line[100 + 1];
	int32_t cnt;
	int32_t matnum;
	BOOL fOverload = FALSE;
	char entryname[_MAX_PATH];
	C4Group* base = &hGroup;

	if(OverloadFile != NULL)
	{
		if(!OverloadFile->LoadEntry("TexMap.txt", &filebuf, NULL, 1) )
			throw std::runtime_error(std::string("Could not read TexMap.txt from group ") + OverloadFile->GetName() + ": " + OverloadFile->GetError() );

		// The first LoadEntry call seems to retrieve incorrect
		// results sometimes
		delete[] filebuf;
		if(!OverloadFile->LoadEntry("TexMap.txt", &filebuf, NULL, 1) )
			throw std::runtime_error(std::string("Could not read TexMap.txt from group ") + OverloadFile->GetName() + ": " + OverloadFile->GetError() );

		for(cnt = 0; SCopySegment((char*)filebuf, cnt, line, 0x0A, 100); ++ cnt)
		{
			SReplaceChar(line, 0x0D, 0x00);
			if(SEqual2(line, "OverloadMaterials") )
			{
				fOverload = TRUE;
				break;
			}
		}

		delete[] filebuf;
	}

	if(fOverload == FALSE && OverloadFile)
	{
		base = OverloadFile;
		OverloadFile = NULL;
	}

	matnum = hGroup.EntryCount(C4CFN_MaterialFiles);
	if(OverloadFile)
	{
		OverloadFile->ResetSearch();
		while(OverloadFile->FindNextEntry(C4CFN_MaterialFiles, entryname) )
			if(!base->FindEntry(entryname) )
				++ matnum;
	}

	Map = new C4Material[matnum];
	base->ResetSearch();

	cnt = 0;
	while(base->FindNextEntry(C4CFN_MaterialFiles, entryname) )
	{
		if(OverloadFile)
			if(OverloadFile->FindEntry(entryname) )
				continue;

		try
		{
			Map[cnt ++].Load(*base, entryname);
		}
		catch(...)
		{
			delete[] Map;
			throw;
		}
	}

	if(OverloadFile)
	{
		OverloadFile->ResetSearch();
		while(OverloadFile->FindNextEntry(C4CFN_MaterialFiles, entryname) )
		{
			try
			{
				Map[cnt ++].Load(*OverloadFile, entryname);
			}
			catch(...)
			{
				delete[] Map;
				throw;
			}
		}
	}

	Num = cnt;
}

int32_t C4MaterialMap::Get(const char* mat_name)
{
	int32_t cnt;
	for(cnt = 0; cnt < Num; ++ cnt)
		if(SEqualNoCase(mat_name, Map[cnt].Name) )
			return cnt;

	return MNone;
}
