// Simple C4TextureMap for use by standalone C4MapCreatorS2

#include <stdio.h>
#include <stdexcept>
#include <Standard.h>
#include <C4Constants.h>
#include <C4Components.h>
#include <C4Texture.h>

C4TextureMap::C4TextureMap(C4Group& hGroup, C4Group* OverloadFile)
{
	load_group(hGroup, OverloadFile);
}

void C4TextureMap::load_group(C4Group& hGroup, C4Group* OverloadFile)
{
	uint8_t* filebuf;
	char line[100 + 1];
	char texname[_MAX_PATH + 1];
	int32_t cnt;
	BOOL has_Overload = FALSE;

	// Read TexMap.txt for "OverloadTextures" entry
	if(OverloadFile != NULL)
	{
		if(!OverloadFile->LoadEntry("TexMap.txt", &filebuf, NULL, 1) )
			throw std::runtime_error(std::string("Could not read TexMap.txt from group ") + OverloadFile->GetName() );

		for(cnt = 0; SCopySegment((char*)filebuf, cnt, line, 0x0A, 100); ++ cnt)
		{
			SReplaceChar(line, 0x0D, 0x00);
			if(SEqual2(line, "OverloadTextures") )
			{
				has_Overload = TRUE;
				break;
			}
		}

		load_group(*OverloadFile, NULL);
		delete[] filebuf;
	}

	if(has_Overload == FALSE && OverloadFile) return;

	// Prefer PNG
	hGroup.ResetSearch();
	while(hGroup.AccessNextEntry(C4CFN_PNGFiles, NULL, texname) )
	{
		SReplaceChar(texname, '.', '\0');
		if(!CheckTexture(texname) )
			m_textures.push_back(texname);
	}

	hGroup.ResetSearch();
	while(hGroup.AccessNextEntry(C4CFN_BitmapFiles, NULL, texname) )
	{
		SReplaceChar(texname, '.', '\0');
		if(!CheckTexture(texname) )
			m_textures.push_back(texname);
	}
}

bool C4TextureMap::CheckTexture(const char* tex_name)
{
	return GetIndex(tex_name, FALSE) != 0;
/*	std::vector<std::string>::iterator iter =
		std::find(m_textures.begin(), m_textures.end(), std::string(tex_name) );

	return iter != m_textures.end();*/
}

BYTE C4TextureMap::GetIndex(const char* tex_name, BOOL add_if_not_exist)
{
	// Index is position in list + 1, 0 is invalid index
	std::vector<std::string>::size_type i;
	for(i = 0; i < m_textures.size(); ++ i)
		if(stricmp(m_textures[i].c_str(), tex_name) == 0)
			return i + 1;

	if(add_if_not_exist)
	{
		m_textures.push_back(tex_name);
		return m_textures.size();
	}

	return 0;
}
