
#include <Standard.h>
#include <StdBuf.h>
#include <StdCompiler.h>
#include <StdAdaptors.h>
#include <StdFile.h>

#include <stdarg.h>
#include <stdio.h>
#ifdef _WIN32
#include <io.h>
#define vsnprintf _vsnprintf
#else
#define O_BINARY 0
#define O_SEQUENTIAL 0
#include <unistd.h>
#include <stdlib.h>
#include <ctype.h>
#endif
#include <fcntl.h>
#include <sys/stat.h>


// *** StdBuf

bool StdBuf::LoadFromFile(const char *szFile)
{
  // Open file
  int fh = open(szFile, O_BINARY | O_RDONLY | O_SEQUENTIAL, S_IREAD | S_IWRITE);
  if(fh < 0) return false;
  // Create buf
  New(FileSize(fh));
  // Read
  if(read(fh, getMData(), getSize()) != (signed int) getSize())
  {
    close(fh);
    return false;
  }
  close(fh);
  // Ok
  return true;
}
bool StdBuf::SaveToFile(const char *szFile) const
{
  // Open file
  int fh = open(szFile, O_BINARY | O_CREAT | O_WRONLY | O_SEQUENTIAL | O_TRUNC, S_IREAD | S_IWRITE);
  if(fh < 0) return false;
  // Write data
  if(write(fh, getData(), getSize()) != (signed int) getSize())
  {
    close(fh);
    return false;
  }
  close(fh);
  // Ok
  return true;
}

void StdBuf::CompileFunc(StdCompiler *pComp, int iType)
{
  // Write size & raw data
  pComp->Value(mkIntAdapt(iSize));
  pComp->Seperator(StdCompiler::SEP_PART2);
  if(pComp->isCompiler())
  {
    New(iSize);
    pComp->Raw(getMData(), iSize, StdCompiler::RawCompileType(iType));
  }
  else
  {
    pComp->Raw(const_cast<void *>(getData()), iSize, StdCompiler::RawCompileType(iType));
  }
}

// *** StdStringBuf

void StdStrBuf::Format(const char *szFmt, ...)
{
	// Create argument list
  va_list args; va_start(args, szFmt);
	// Format
	FormatV(szFmt, args);
}

void StdStrBuf::FormatV(const char *szFmt, va_list args)
{
	// Clear previous contents
	Clear();
	// Format
	AppendFormatV(szFmt, args);
}

void StdStrBuf::AppendFormat(const char *szFmt, ...)
{
	// Create argument list
  va_list args; va_start(args, szFmt);
	// Format
	AppendFormatV(szFmt, args);
}

void StdStrBuf::AppendFormatV(const char *szFmt, va_list args)
{
	// Save append start
	int iStart = getLength();
#ifdef HAVE_VSCPRINTF
  // Calculate size, allocate
  int iLength = _vscprintf(szFmt, args);
  Grow(iLength);
  // Format
  char *pPos = getMElem<char>(*this, iSize - iLength - 1);
  vsprintf(getMPtr(iStart), szFmt, args);
#else
	int iBytes;
	do
	{
		// Grow
		Grow(512);
		// Try output
		iBytes = vsnprintf(getMPtr(iStart), getLength(), szFmt, args);
	}
	while(iBytes < 0 || (unsigned int)(iBytes) >= getLength());
	// Calculate real length, if vsnprintf didn't return anything of value
	iBytes = strlen(getData());
	// Shrink to fit
	SetSize(iBytes + 1);
#endif
}

void StdStrBuf::CompileFunc(StdCompiler *pComp, int iRawType)
{
  if(pComp->isCompiler())
  {
    char *pnData;
    pComp->String(&pnData, StdCompiler::RawCompileType(iRawType));
    Take(pnData);
  }
  else
  {
    char *pData = const_cast<char *>(getData());
    pComp->String(&pData, StdCompiler::RawCompileType(iRawType));
  }
}

StdStrBuf FormatString(const char *szFmt, ...)
{
  va_list args; va_start(args, szFmt);
  return FormatStringV(szFmt, args);
}

StdStrBuf FormatStringV(const char *szFmt, va_list args)
{
  StdStrBuf Buf;
  Buf.FormatV(szFmt, args);
  return Buf;
}
