
#include "StdCompiler.h"

#include <stdio.h> 
#include <ctype.h>

// *** StdCompilerBinWrite

void StdCompilerBinWrite::DWord(int32_t &rInt)   { WriteValue(rInt); }
void StdCompilerBinWrite::DWord(uint32_t &rInt)  { WriteValue(rInt); }
void StdCompilerBinWrite::Word(int16_t &rShort)  { WriteValue(rShort); }
void StdCompilerBinWrite::Word(uint16_t &rShort) { WriteValue(rShort); }
void StdCompilerBinWrite::Byte(int8_t &rByte)    { WriteValue(rByte); }
void StdCompilerBinWrite::Byte(uint8_t &rByte)   { WriteValue(rByte); }
void StdCompilerBinWrite::Boolean(bool &rBool)   { WriteValue(rBool); }
void StdCompilerBinWrite::Character(char &rChar) { WriteValue(rChar); }
void StdCompilerBinWrite::String(char *szString, size_t iMaxLength, RawCompileType eType)
{
  WriteData(szString, strlen(szString) + 1);
}
void StdCompilerBinWrite::String(char **pszString, RawCompileType eType)
{
  if(*pszString)
    WriteData(*pszString, strlen(*pszString) + 1);
  else
    WriteValue('\0');
}

template <class T> 
  void StdCompilerBinWrite::WriteValue(const T &rValue)
	{
		// Copy data
		if(fSecondPass)
			*getMBufPtr<T>(Buf, iPos) = rValue;
		iPos += sizeof(rValue);
	}

void StdCompilerBinWrite::WriteData(const void *pData, size_t iSize)
{
	// Copy data
  if(fSecondPass)
		Buf.Write(pData, iSize, iPos);
  iPos += iSize;
}

void StdCompilerBinWrite::Raw(void *pData, size_t iSize, RawCompileType eType)
{
	// Copy data
  if(fSecondPass)
		Buf.Write(pData, iSize, iPos);
  iPos += iSize;
}

void StdCompilerBinWrite::Begin()
{
  fSecondPass = false; iPos = 0;
}

void StdCompilerBinWrite::BeginSecond()
{
  Buf.New(iPos);
  fSecondPass = true; iPos = 0;
}

// *** StdCompilerBinRead

void StdCompilerBinRead::DWord(int32_t &rInt)   { ReadValue(rInt); }
void StdCompilerBinRead::DWord(uint32_t &rInt)  { ReadValue(rInt); }
void StdCompilerBinRead::Word(int16_t &rShort)  { ReadValue(rShort); }
void StdCompilerBinRead::Word(uint16_t &rShort) { ReadValue(rShort); }
void StdCompilerBinRead::Byte(int8_t &rByte)    { ReadValue(rByte); }
void StdCompilerBinRead::Byte(uint8_t &rByte)   { ReadValue(rByte); }
void StdCompilerBinRead::Boolean(bool &rBool)   { ReadValue(rBool); }
void StdCompilerBinRead::Character(char &rChar) { ReadValue(rChar); }
void StdCompilerBinRead::String(char *szString, size_t iMaxLength, RawCompileType eType)
{
	// At least one byte data needed
	if(iPos >= Buf.getSize())
		throw new EOFException(iPos);
	// Copy until no data left
	while(*szString++ = *getBufPtr<char>(Buf, iPos++))
		if(iPos >= Buf.getSize())
		  throw new EOFException(iPos);
}

void StdCompilerBinRead::String(char **pszString, RawCompileType eType)
{
	// At least one byte data needed
	if(iPos >= Buf.getSize())
		throw new EOFException(iPos);
	int iStart = iPos;
	// Search string end
	while(*getBufPtr<char>(Buf, iPos++))
		if(iPos >= Buf.getSize())
		  throw new EOFException(iPos);
	// Allocate and copy data
	*pszString = new char [iPos - iStart];
	memcpy(*pszString, Buf.getPtr(iStart), iPos - iStart);
}

void StdCompilerBinRead::Raw(void *pData, size_t iSize, RawCompileType eType)
{
  if(iPos + iSize > Buf.getSize())
		throw new EOFException(iPos);
  // Copy data
  memcpy(pData, Buf.getPtr(iPos), iSize);
  iPos += iSize;
}


template <class T>
	inline void StdCompilerBinRead::ReadValue(T &rValue)
	{
		// Pufferberhang prfen
		if(iPos + sizeof(T) > Buf.getSize())
 		  throw new EOFException(iPos);
		// Kopieren
		rValue = *getBufPtr<T>(Buf, iPos);
		iPos += sizeof(T);
	}

void StdCompilerBinRead::Begin()
{
	iPos = 0;
}

// *** StdCompilerINIWrite

bool StdCompilerINIWrite::Name(const char *szName)
{
  // Sub-Namesections exist, so it's a section. Write name if not already done so.
  if(fPutName) PutName(true);
  // Push struct
  Naming *pnNaming = new Naming;
  pnNaming->Name.Copy(szName);
  pnNaming->Parent = pNaming;
  pNaming = pnNaming;
  iDepth++;
  // Safe back the name
  fPutName = true; fInSection = false;
  return true;
}

void StdCompilerINIWrite::NameEnd()
{
  // Nothing written? Put name anyway
  if(fPutName) PutName(false);
  // Pop
  assert(iDepth);
  Naming *poNaming = pNaming;
  pNaming = poNaming->Parent;
  delete poNaming;
  iDepth--;
  // We're inside a section now
  fInSection = true;
  // Append newline
  Buf.Append("\r\n");
}

bool StdCompilerINIWrite::FollowName(const char *szName)
{
	// End current naming
	NameEnd();
	// Start new one
	Name(szName);
	// Done
	return true;
}

bool StdCompilerINIWrite::Seperator(Sep eSep)
{
  if(fInSection)
  {
    // Re-put section name
    Buf.Append("\r\n");
    PutName(true);
  }
  else
  {
    switch(eSep)
    {
    case SEP_SEP: Buf.AppendChar(','); break;
    case SEP_SEP2: Buf.AppendChar(';'); break;
    case SEP_SET: Buf.AppendChar('='); break;
    case SEP_PART: Buf.AppendChar('.'); break;
    case SEP_PART2: Buf.AppendChar(':'); break;
		case SEP_PLUS: Buf.AppendChar('+'); break;
		case SEP_START: Buf.AppendChar('('); break;
		case SEP_END: Buf.AppendChar(')'); break;
		case SEP_START2: Buf.AppendChar('['); break;
		case SEP_END2: Buf.AppendChar(']'); break;
    default: assert(false);
    }
  }
  return true;
}

void StdCompilerINIWrite::DWord(int32_t &rInt)
{
  PrepareForValue();
  Buf.AppendFormat("%d", rInt);
}
void StdCompilerINIWrite::DWord(uint32_t &rInt)
{
  PrepareForValue();
  Buf.AppendFormat("%u", rInt);
}
void StdCompilerINIWrite::Word(int16_t &rInt)
{
  PrepareForValue();
  Buf.AppendFormat("%d", rInt);
}
void StdCompilerINIWrite::Word(uint16_t &rInt)
{
  PrepareForValue();
  Buf.AppendFormat("%u", rInt);
}
void StdCompilerINIWrite::Byte(int8_t &rByte)
{
  PrepareForValue();
  Buf.AppendFormat("%d", rByte);
}
void StdCompilerINIWrite::Byte(uint8_t &rInt)
{
  PrepareForValue();
  Buf.AppendFormat("%u", rInt);
}
void StdCompilerINIWrite::Boolean(bool &rBool)
{
  PrepareForValue();
  Buf.Append(rBool ? "true" : "false");
}
void StdCompilerINIWrite::Character(char &rChar)
{
  PrepareForValue();
  Buf.AppendFormat("%c", rChar);
}

void StdCompilerINIWrite::String(char *szString, size_t iMaxLength, RawCompileType eType)
{
  PrepareForValue();
  switch(eType)
  {
  case RCT_Escaped:
    WriteEscaped(szString, szString + strlen(szString));
    break;
  case RCT_All:
  case RCT_Idtf:
  case RCT_ID:
    Buf.Append(szString);
  }
}

void StdCompilerINIWrite::String(char **pszString, RawCompileType eType)
{
  char cNull = '\0';
  String(*pszString ? *pszString : &cNull, 0, eType);
}

void StdCompilerINIWrite::Raw(void *pData, size_t iSize, RawCompileType eType)
{
  switch(eType)
  {
  case RCT_Escaped:
    WriteEscaped(reinterpret_cast<char *>(pData), reinterpret_cast<char *>(pData) + iSize);
    break;
  case RCT_All:
  case RCT_Idtf:
  case RCT_ID:
    Buf.Append(reinterpret_cast<char *>(pData), iSize);
  }
}


void StdCompilerINIWrite::Begin()
{
  pNaming = NULL;
  fPutName = false;
  iDepth = 0;
  fInSection = false;
  Buf.Clear();
}

void StdCompilerINIWrite::End()
{
  // Truncate whitespace
  unsigned int iTrunc = 0; const char *pPos = Buf.getPtr(Buf.getLength() - 1);
  while(iTrunc < Buf.getLength() && (*pPos == ' ' || *pPos == '\n' || *pPos == '\r'))
    { iTrunc++; pPos--; }
  if(iTrunc) Buf.Shrink(iTrunc);
  // Ensure all namings were closed properly
  assert(!iDepth);
}

void StdCompilerINIWrite::PrepareForValue()
{
  // Put name (value-type), if not already done so
  if(fPutName) PutName(false);
  // No data allowed inside of sections
  assert(!fInSection);
  // No values allowed on top-level - must be contained in at least one section
  assert(iDepth > 1);
}

void StdCompilerINIWrite::WriteEscaped(const char *szString, const char *pEnd)
{
  Buf.AppendChar('"');
  // Try to write chunks as huge as possible of "normal" chars.
  // Note this excludes '\0', so the standard Append() can be used.
  const char *pStart, *pPos; pStart = pPos = szString;
  bool fLastNumEscape = false; // catch "\1""1", which must become "\1\61"
  for(; pPos < pEnd; pPos++)
    if(!isprint(*pPos) || *pPos == '\\' || *pPos == '"' || (fLastNumEscape && isdigit(*pPos)))
    {
      // Write everything up to this point
      Buf.Append(pStart, pPos - pStart);
      // Escape
      fLastNumEscape = false;
      switch(*pPos)
      {
      case '\a': Buf.Append("\\a"); break;
      case '\b': Buf.Append("\\b"); break;
      case '\f': Buf.Append("\\f"); break;
      case '\n': Buf.Append("\\n"); break;
      case '\r': Buf.Append("\\r"); break;
      case '\t': Buf.Append("\\t"); break;
      case '\v': Buf.Append("\\v"); break;
      case '\"': Buf.Append("\\\""); break;
      case '\\': Buf.Append("\\\\"); break;
      default:
        Buf.AppendFormat("\\%o", *reinterpret_cast<const unsigned char *>(pPos));
        fLastNumEscape = true;
      }
      // Set pointer
      pStart = pPos + 1;
    }
    else
      fLastNumEscape = false;
  // Write the rest
  Buf.Append(pStart, pEnd - pStart);
  Buf.AppendChar('"');
}

void StdCompilerINIWrite::WriteIndent(bool fSection)
{
  // Do not indent level 1 (level 0 values aren't allowed - see above)
  int iIndent = iDepth - 1;
  // Sections are indented more, even though they belong to this level
  if(!fSection) iIndent--;
  // Do indention
  if(iIndent <= 0) return;
  Buf.AppendChars(' ',  iIndent * 2);
}

void StdCompilerINIWrite::PutName(bool fSection)
{
  WriteIndent(fSection);
  // Put name
  if(fSection)
    Buf.AppendFormat("[%s]\r\n", pNaming->Name.getData());
  else
    Buf.AppendFormat("%s=", pNaming->Name.getData());
  // Set flag
  fPutName = false;
}

// *** StdCompilerINIRead

StdCompilerINIRead::WarnCBT StdCompilerINIRead::pWarnCB = NULL;

StdCompilerINIRead::StdCompilerINIRead()
  : pNameRoot(NULL), iDepth(0), iRealDepth(0)
{

}

// Naming
bool StdCompilerINIRead::Name(const char *szName)
{
  // Increase depth
  iDepth++;
  // Parent category virtual?
  if(iDepth - 1 > iRealDepth)
    return false;
  // Name must be alphanumerical and non-empty (force it)
  if(!isalpha(*szName))
    { assert(false); return false; }
  for(const char *p = szName + 1; *p; p++)
    if(!isalnum(*p) && *p != ' ' && *p != '_')
      { assert(false); return false; }
  // Search name
  NameNode *pNode;
  for(pNode = pName->FirstChild; pNode; pNode = pNode->NextChild)
    if(pNode->Pos && pNode->Name == szName)
      break;
  // Not found?
  if(!pNode) return false;
  // Save tree position, indicate success
  pName = pNode;
  pPos = pName->Pos;
	pReenter = NULL;
  iRealDepth++;
  return true;
}
void StdCompilerINIRead::NameEnd()
{
  assert(iDepth > 0);
  if(iRealDepth == iDepth)
  {
    // Report unused entries
    for(NameNode *pNode = pName->FirstChild; pNode; pNode = pNode->NextChild)
      if(pNode->Pos)
        if(pNode->Section)
          Warn("Unknown section \"%s\"!", pNode->Name.getData());
        else
          Warn("Unknown value \"%s\" in section \"%s\"!", pNode->Name.getData(), pName->Name.getData());
    // Reset position so this name won't be found again
    pName->Pos = NULL;    
    // Go up
    pName = pName->Parent;
    iRealDepth--;
  }
  // Decrease depth
  iDepth--;
  // This is the middle of nowhere
  pPos = NULL; pReenter = NULL;
}

bool StdCompilerINIRead::FollowName(const char *szName)
{
	// Current naming virtual?
  if(iDepth > iRealDepth)
    return false;
	// Next section must be the one
	if(!pName->NextChild || pName->NextChild->Name != szName)
	{
		// End current naming
		NameEnd();
		// Go into virtual naming
		iDepth++;
		return false;
	}
	// End current naming
	NameEnd();
	// Start new one
	Name(szName);
	// Done
	return true;
}

// Seperators
bool StdCompilerINIRead::Seperator(Sep eSep)
{
  if(iDepth > iRealDepth) return false;
  // In section?
  if(pName->Section)
  {
    // Store current name, search another section with the same name
    StdStrBuf CurrName = pName->Name;
    NameEnd();
    return Name(CurrName.getData());
  }
	// Position saved back from seperator mismatch?
  if(pReenter) { pPos = pReenter; pReenter = NULL; }
	// Nothing to read?
	if(!pPos) return false;
  // Seperator type
  char cSep;
  switch(eSep)
  {
  case SEP_SEP: cSep = ','; break;
  case SEP_SEP2: cSep = ';'; break;
  case SEP_SET: cSep = '='; break;
  case SEP_PART: cSep = '.'; break;
  case SEP_PART2: cSep = ':'; break;
	case SEP_PLUS: cSep = '+'; break;
	case SEP_START: cSep = '('; break;
	case SEP_END: cSep = ')'; break;
	case SEP_START2: cSep = '['; break;
	case SEP_END2: cSep = ']'; break;
  default: assert(false);
  }
  // Read (while skipping over whitespace)
  SkipWhitespace();
	// Seperator mismatch? Let all read attempts fail until the correct seperator is found or the naming ends.
  if(*pPos != cSep) { pReenter = pPos; pPos = NULL; return false; }
  // Go over seperator, success
  pPos++;
  return true;
}

int StdCompilerINIRead::GetSubSectionCount(const char *szName)
	{
	// not in virtual naming
	if (iDepth > iRealDepth || !pName) return 0;
	// count within current name
	int iCount = 0;
  NameNode *pNode;
  for(pNode = pName->FirstChild; pNode; pNode = pNode->NextChild)
		// if no name is given, all valid subsections are counted
    if(pNode->Pos && (!szName || pNode->Name == szName))
      ++iCount;
	return iCount;
	}

// Various data readers
void StdCompilerINIRead::DWord(int32_t &rInt)
{
  rInt = ReadNum();
}
void StdCompilerINIRead::DWord(uint32_t &rInt)
{
  rInt = ReadUNum();
}
void StdCompilerINIRead::Word(int16_t &rShort)
{
  const int MIN = -(1 << 15), MAX = (1 << 15) - 1;
  int iNum = ReadNum();
  if(iNum < MIN || iNum > MAX)
    Warn("number out of range (%d to %d): %d ", MIN, MAX, iNum);
  rShort = BoundBy(iNum, MIN, MAX);
}
void StdCompilerINIRead::Word(uint16_t &rShort)
{
  const unsigned int MIN = 0, MAX = (1 << 15) - 1;
  unsigned int iNum = ReadUNum();
  if(iNum > MAX)
    Warn("number out of range (%u to %u): %u ", MIN, MAX, iNum);
  rShort = BoundBy(iNum, MIN, MAX);
}
void StdCompilerINIRead::Byte(int8_t &rByte)
{
  const int MIN = -(1 << 7), MAX = (1 << 7) - 1;
  int iNum = ReadNum();
  if(iNum < MIN || iNum > MAX)
    Warn("number out of range (%d to %d): %d ", MIN, MAX, iNum);
  rByte = BoundBy(iNum, MIN, MAX);
}
void StdCompilerINIRead::Byte(uint8_t &rByte)
{
  const unsigned int MIN = 0, MAX = (1 << 7) - 1;
  unsigned int iNum = ReadUNum();
  if(iNum > MAX)
    Warn("number out of range (%u to %u): %u ", MIN, MAX, iNum);
  rByte = BoundBy(iNum, MIN, MAX);
}
void StdCompilerINIRead::Boolean(bool &rBool)
{
  if(!pPos)
    throw new NotFoundException(0, "boolean expected");
  if(*pPos == '1' && !isdigit(*(pPos+1)))
    { rBool = true; pPos ++; }
  else if(*pPos == '0' && !isdigit(*(pPos+1)))
    { rBool = false; pPos ++; }
  else if(SEqual2(pPos, "true"))
    { rBool = true; pPos += 4; }
  else if(SEqual2(pPos, "false"))
    { rBool = false; pPos += 5; }
  else
    throw new NotFoundException(pPos - Buf.getData(), "boolean expected");
}
void StdCompilerINIRead::Character(char &rChar)
{
  if(!pPos)
    throw new NotFoundException(0, "character expected");
  if(!isalpha(*pPos))
    throw new NotFoundException(pPos - Buf.getData(), "character expected");
  rChar = *pPos++;
}
void StdCompilerINIRead::String(char *szString, size_t iMaxLength, RawCompileType eType)
{
  // Read data
  StdBuf Buf = ReadString(iMaxLength, eType, true);
  // Copy
  SCopy(getBufPtr<char>(Buf), szString, iMaxLength);
}
void StdCompilerINIRead::String(char **pszString, RawCompileType eType)
{
  // For Backwards compatibility: Escaped strings default to normal strings if no escaped string is given
	if (eType == RCT_Escaped && pPos && *pPos!='"') eType = RCT_All;
  // Get length
  size_t iLength = GetStringLength(eType);
  // Read data
  StdBuf Buf = ReadString(iLength, eType, true);
  // Set
  *pszString = reinterpret_cast<char *>(Buf.GrabPointer());
}
void StdCompilerINIRead::Raw(void *pData, size_t iSize, RawCompileType eType)
{
  // Read data
  StdBuf Buf = ReadString(iSize, eType, false);
  // Correct size?
  if(Buf.getSize() != iSize)
    Warn("got %u bytes raw data, but %u bytes expected!", Buf.getSize(), iSize);
  // Copy
  MemCopy(Buf.getData(), pData, iSize);
}

void StdCompilerINIRead::Begin()
{
  assert(!Buf.isNull());
  // Already running? This may happen if someone confuses Compile with Value.
  assert(!iDepth && !iRealDepth && !pNameRoot);
  // Create tree
  CreateNameTree();
  // Start must be inside a section
  iDepth = iRealDepth = 0;
  pPos = NULL; pReenter = NULL;
}
void StdCompilerINIRead::End()
{
  assert(!iDepth && !iRealDepth);
  FreeNameTree();
}

void StdCompilerINIRead::CreateNameTree()
{
  // Create root node
  pName = pNameRoot = new NameNode();
  // Start scanning
  const char *pPos = Buf.getPtr(0);
  while(*pPos)
  {
    // Go over whitespace
    int iIndent = 0;
    while(*pPos == ' ' || *pPos == '\t')
      { pPos++; iIndent++; }
    // Name/Section?
    bool fSection = *pPos == '[' && isalpha(*(pPos+1));
    if(fSection || isalpha(*pPos))
    {
      // Treat values as if they had more indention
      // (so they become children of sections on the same level)
      if(!fSection) iIndent++; else pPos++;
      // Go up in tree structure if there is less indention
      while(pName->Parent && pName->Indent >= iIndent)
        pName = pName->Parent;
      // Copy name
      StdStrBuf Name;
      while(isalnum(*pPos) || *pPos == ' ' || *pPos == '_')
        Name.AppendChar(*pPos++);
      while(isspace(*pPos)) pPos++;
      if( *pPos != (fSection ? ']' : '=') )
        // Warn, ignore
        Warn(isprint(*pPos) ? "Unexpected character ('%c'): %s ignored" : "Unexpected character ('0x%2x'): %s ignored", unsigned(*pPos), fSection ? "section" : "value");
      else
      {
        pPos++;
        // Create new node
        pName =
          pName->LastChild = 
            (pName->LastChild ? pName->LastChild->NextChild :  pName->FirstChild) = 
              new NameNode(pName);
        pName->Name.Take(Name);
        pName->Pos = pPos;
        pName->Indent = iIndent;
        pName->Section = fSection;
        // Values don't have children (even if the indention looks like it)
        if(!fSection)
          pName = pName->Parent;
      }
    }
    // Skip line
    while(*pPos && (*pPos != '\n' && *pPos != '\r'))
      pPos++;
    while(*pPos == '\n' || *pPos == '\r')
      pPos++;
  }
  // Set pointer back
  pName = pNameRoot;
}

void StdCompilerINIRead::FreeNameTree()
{
  // free all nodes
  NameNode *pNode = pNameRoot;
  while(pNode)
  {
    if(pNode->FirstChild)
      (pNode = pNode->FirstChild)->Parent->FirstChild = NULL;
    else
    {
      NameNode *pDelete = pNode;
      pNode = (pNode->NextChild ? pNode->NextChild : pNode->Parent);
      delete pDelete;
    }
  }
  pName = pNameRoot = NULL;
}

void StdCompilerINIRead::SkipWhitespace()
{
  while(*pPos == ' ' || *pPos == '\t')
    pPos++;
}

void StdCompilerINIRead::SkipNum()
{
  while(*pPos == '+' || *pPos == '-' || isdigit(*pPos))
    pPos++;
}

int StdCompilerINIRead::ReadNum()
{
  if(!pPos)
    throw new NotFoundException(0, "number expected");
  // Skip whitespace
  SkipWhitespace();
  // Read
  int iNum;
  if(sscanf(pPos, "%d", &iNum) != 1)
    throw new NotFoundException(pPos - Buf.getData(), "number expected");
  // Get over it
  SkipNum();
  return iNum;
}

unsigned int StdCompilerINIRead::ReadUNum()
{
  if(!pPos)
    throw new NotFoundException(0, "number expected");
  // Skip whitespace
  SkipWhitespace();
  // Read
  unsigned int iNum;
  if(sscanf(pPos, "%u", &iNum) != 1)
    throw new NotFoundException(pPos - Buf.getData(), "number expected");
  // Get over it
  SkipNum();
  return iNum;
}

size_t StdCompilerINIRead::GetStringLength(RawCompileType eRawType)
{
  // Excpect valid position
  if(!pPos)
    throw new NotFoundException(0, "string expected");
  // Skip whitespace
  SkipWhitespace();
  // Save position
  const char *pStart = pPos;
  // Escaped? Go over '"'
  if(eRawType == RCT_Escaped && *pPos++ != '"')
    throw new NotFoundException(pPos - Buf.getData() - 1, "escaped string expected");
  // Search end of string
  size_t iLength = 0;
  while(!TestStringEnd(eRawType))
  {
    // Read a character (we're just counting atm)
    if(eRawType == RCT_Escaped)
      ReadEscapedChar();
    else
      pPos++;
    // Count it
    iLength++;
  }
  // Reset position, return the length  
  pPos = pStart;
  return iLength;
}

StdBuf StdCompilerINIRead::ReadString(size_t iLength, RawCompileType eRawType, bool fAppendNull)
{
  // Excpect valid position
  if(!pPos)
    throw new NotFoundException(0, "string expected");
  // Skip whitespace
  SkipWhitespace();
  // Escaped? Go over '"'
  if(eRawType == RCT_Escaped && *pPos++ != '"')
    throw new NotFoundException(pPos - Buf.getData() - 1, "escaped string expected");
  // Create buffer
  StdBuf OutBuf; OutBuf.New(iLength + (fAppendNull ? sizeof('\0') : 0));
  // Read
  char *pOut = getMBufPtr<char>(OutBuf);
  while(iLength && !TestStringEnd(eRawType))
  {
    // Read a character
    if(eRawType == RCT_Escaped)
      *pOut++ = ReadEscapedChar();
    else
      *pOut++ = *pPos++;
    // Count it
    iLength--;
  }
  // Escaped: Go over '"'
  if(eRawType == RCT_Escaped)
  {
    while(*pPos != '"')
    {
      if(!*pPos || *pPos == '\n' || *pPos == '\r')
        throw new EOFException(pPos - Buf.getData(), "end of file while searching for string end (\'\"\')");
      pPos++;
    }
    pPos++;
  }
  // Append null
  if(fAppendNull)
    *pOut = '\0';
  // Shrink, if less characters were read
  OutBuf.Shrink(iLength);
  // Done
  return OutBuf;
}

bool StdCompilerINIRead::TestStringEnd(RawCompileType eType)
{
  switch(eType)
  {
  case RCT_Escaped: return *pPos == '"';
  case RCT_All: return !*pPos || *pPos == '\n' || *pPos == '\r';
  // '-' is needed for Layers in Scenario.txt (C4NameList)
  case RCT_Idtf: case RCT_ID: return !isalnum(*pPos) && *pPos != '_' && *pPos != '-';
  }
  // unreachable
  return true;
}

char StdCompilerINIRead::ReadEscapedChar()
{
  // Catch some no-noes like \0, \n etc.
  if(!isprint(*pPos)) 
    throw new CorruptException(pPos - Buf.getData() - 1, "nonprintable character found in string: %02x", unsigned(*pPos));
  // Not escaped? Just return it
  if(*pPos != '\\') return *pPos++;
  // What type of escape?
  switch(*++pPos)
  {
  case 'a': pPos++; return '\a';
  case 'b': pPos++; return '\b';
  case 'f': pPos++; return '\f';
  case 'n': pPos++; return '\n';
  case 'r': pPos++; return '\r';
  case 't': pPos++; return '\t';
  case 'v': pPos++; return '\v';
  case '\'': pPos++; return '\'';
  case '"': pPos++; return '"';
  case '\\': pPos++; return '\\';
  case '?': pPos++; return '?';
  case 'x':
    // Treat '\x' as 'x' - damn special cases
    if(!isxdigit(*++pPos))
      return 'x';
    else
    {
      // Read everything that looks like it might be hexadecimal - MSVC does it this way, so do not sue me.
      int iCode = 0;
      do
        { iCode = iCode * 16 + (isdigit(*pPos) ? *pPos - '0' : *pPos - 'a' + 10); pPos++; }
      while(isxdigit(*pPos));
      // Done. Don't bother to check the range (we aren't doing anything mission-critical here, are we?)
      return char(iCode);
    }
  default:
    // Not octal? Let it pass through.
    if(!isdigit(*pPos) || *pPos >= '8')
      return *pPos++;
    else
    {
      // Read it the octal way.
      int iCode = 0;
      do
        { iCode = iCode * 8 + (*pPos - '0'); pPos++;}
      while(isdigit(*pPos) && *pPos < '8');
      // Done. See above.
      return char(iCode);
    }
  }
  // unreachable
	assert (false);
}

void StdCompilerINIRead::Warn(const char *szWarning, ...)
{
  // Got warning callback?
  if(!pWarnCB) return;
  // Format message
  va_list args; va_start(args, szWarning);
  StdStrBuf Msg; Msg.FormatV(szWarning, args);
  // do callback
  (*pWarnCB)(pPos ? NULL : pPos - Buf.getData(), Msg.getData());
}
